/*
 * Copyright (c) 1999, Jesper Hansen. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither name of the company nor the names of its contributors may
 *    be used to endorse or promote products derived from this software
 *    without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

//-----------------------------------------------------------------------------
//-------------------------------------
#include <interface/View.h>
//-------------------------------------
#include "dle/Core.h"
//#include "gfx/BitmapScale.h"
#include "misc/AutoPtr.h"
//-----------------------------------------------------------------------------

class BBitmap;

class SelectionBitmapView : public BView, public dle::Object
{
public:
	SelectionBitmapView( bool selectable=false );
	~SelectionBitmapView();

	void SetBitmap( damn::AutoPtr<BBitmap> bitmap );
	damn::AutoPtr<BBitmap> GetBitmap() const;

	void SetZoomRect( const BRect &rect );
	const BRect &GetZoomRect() const { return fZoomRect; }

	void SetSelectionRect( const BRect &rect );
	BRect GetSelectionRect() const;

	dle::MinMax2 GetMinMaxSize();

	void Draw( BRect area );

private:
	void FrameResized( float new_width, float new_height );

	void MouseDown( BPoint where );
	void MouseUp( BPoint where );
	void MouseMoved( BPoint where, uint32 code, const BMessage *a_message );

protected:
	BRect MakeSelectionRect() const;
	void DrawSelection();

	void ClearSelectionRect();
	void ClampSelectionRect();
	
//	void CalcZoomBitmap();

	damn::AutoPtr<BBitmap>			fBitmap;
	BRect							fZoomRect;
//	damn::AutoPtr<BBitmap>			fZoomedBitmap;
//	damn::bitmapscale_filtertype	fZoomFilter;

	bool	fSelectable;
	
	bool	fDoSelect;
	BPoint	fSelectCorner[2];

	class SelectView;
	SelectView	*fSelectViewsLeft;
	SelectView	*fSelectViewsRight;
	SelectView	*fSelectViewsTop;
	SelectView	*fSelectViewsBottom;
};

//-----------------------------------------------------------------------------
